import json
from abc import ABCMeta, abstractmethod

from aiearth.engine.udf_base.images import Image
from aiearth.engine.udf_base.udf_utils import UDFUtils


class AIEUDAF(metaclass=ABCMeta):
    @abstractmethod
    def __init__(self):
        pass

    @abstractmethod
    def aggregation(self, image: Image, params) -> None:
        pass

    def agg_wrapper(self, image: Image, params_bytes: bytes, file_path: str, content: str) -> None:
        UDFUtils.save_file(file_path, content)
        self.aggregation(image, json.loads(params_bytes.decode('utf-8')))

    @abstractmethod
    def get_result(self) -> dict:
        pass

    def result_wrapper(self, file_path: str, content: str) -> dict:
        UDFUtils.save_file(file_path, content)
        return self.get_result()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

import abc
import inspect
from typing import Union
from types import FunctionType

from aiearth import engine
from aiearth.engine.variable_node import VariableNode
from aiearth.engine.function_node import FunctionNode
from aiearth.engine.customfunction_node import CustomFunctionNode
from aiearth.engine.function_helper import FunctionHelper
from aiearth.core.error.aie_error import AIEError, AIEErrorCode


class ConfusionMatrix(FunctionNode):
    def __init__(self, array: list) -> engine.ConfusionMatrix:
        if array is not None and not isinstance(array, list):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"array 只支持list类型参数, 传入类型为{type(array)}"
            )

        invoke_args = {
            "array": array,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "array" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数array不能为空")

        super(ConfusionMatrix, self).__init__(
            "ConfusionMatrix.constructors", invoke_args
        )

    def accuracy(self) -> object:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("ConfusionMatrix.accuracy", "object", invoke_args)

    def consumersAccuracy(self) -> object:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply(
            "ConfusionMatrix.consumersAccuracy", "object", invoke_args
        )

    def fscore(self, beta: [int, float] = 1.0) -> object:
        if beta is not None and not isinstance(beta, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"beta 只支持(int,float)类型参数, 传入类型为{type(beta)}"
            )

        invoke_args = {
            "input": self,
            "beta": beta,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("ConfusionMatrix.fscore", "object", invoke_args)

    def kappa(self) -> object:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("ConfusionMatrix.kappa", "object", invoke_args)

    def producersAccuracy(self) -> object:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply(
            "ConfusionMatrix.producersAccuracy", "object", invoke_args
        )

    def array(self) -> object:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("ConfusionMatrix.array", "object", invoke_args)

    def labels(self) -> object:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("ConfusionMatrix.labels", "object", invoke_args)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

import abc
import inspect
from typing import Union
from types import FunctionType

from aiearth import engine
from aiearth.engine.variable_node import VariableNode
from aiearth.engine.function_node import FunctionNode
from aiearth.engine.customfunction_node import CustomFunctionNode
from aiearth.engine.function_helper import FunctionHelper
from aiearth.core.error.aie_error import AIEError, AIEErrorCode


class Number(FunctionNode):
    def __init__(self, number: object) -> engine.Number:
        if number is not None and not isinstance(number, object):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"number 只支持object类型参数, 传入类型为{type(number)}"
            )

        invoke_args = {
            "number": number,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "number" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数number不能为空")

        super(Number, self).__init__("Number.constructors", invoke_args)

    def abs(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.abs", "engine.Number", invoke_args)

    def acos(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.acos", "engine.Number", invoke_args)

    def asin(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.asin", "engine.Number", invoke_args)

    def atan(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.atan", "engine.Number", invoke_args)

    def cbrt(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.cbrt", "engine.Number", invoke_args)

    def ceil(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.ceil", "engine.Number", invoke_args)

    def cos(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.cos", "engine.Number", invoke_args)

    def cosh(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.cosh", "engine.Number", invoke_args)

    def exp(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.exp", "engine.Number", invoke_args)

    def floor(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.floor", "engine.Number", invoke_args)

    def log(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.log", "engine.Number", invoke_args)

    def log10(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.log10", "engine.Number", invoke_args)

    def round(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.round", "engine.Number", invoke_args)

    def signum(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.signum", "engine.Number", invoke_args)

    def sin(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.sin", "engine.Number", invoke_args)

    def sinh(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.sinh", "engine.Number", invoke_args)

    def sqrt(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.sqrt", "engine.Number", invoke_args)

    def tan(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.tan", "engine.Number", invoke_args)

    def tanh(self) -> engine.Number:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Number.tanh", "engine.Number", invoke_args)

    def eq(self, right: Union[engine.Number, int, int, float]) -> engine.Number:
        if right is not None and not isinstance(
            right, (engine.Number, int, int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持(engine.Number,int,int,float)类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "input": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Number.eq", "engine.Number", invoke_args)

    def lt(self, right: Union[engine.Number, int, int, float]) -> engine.Number:
        if right is not None and not isinstance(
            right, (engine.Number, int, int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持(engine.Number,int,int,float)类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "input": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Number.lt", "engine.Number", invoke_args)

    def lte(self, right: Union[engine.Number, int, int, float]) -> engine.Number:
        if right is not None and not isinstance(
            right, (engine.Number, int, int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持(engine.Number,int,int,float)类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "input": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Number.lte", "engine.Number", invoke_args)

    def gt(self, right: Union[engine.Number, int, int, float]) -> engine.Number:
        if right is not None and not isinstance(
            right, (engine.Number, int, int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持(engine.Number,int,int,float)类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "input": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Number.gt", "engine.Number", invoke_args)

    def gte(self, right: Union[engine.Number, int, int, float]) -> engine.Number:
        if right is not None and not isinstance(
            right, (engine.Number, int, int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持(engine.Number,int,int,float)类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "input": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Number.gte", "engine.Number", invoke_args)

    def neq(self, right: Union[engine.Number, int, int, float]) -> engine.Number:
        if right is not None and not isinstance(
            right, (engine.Number, int, int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持(engine.Number,int,int,float)类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "input": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Number.neq", "engine.Number", invoke_args)

    def atan2(self, right: Union[engine.Number, int, int, float]) -> engine.Number:
        if right is not None and not isinstance(
            right, (engine.Number, int, int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持(engine.Number,int,int,float)类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "input": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Number.atan2", "engine.Number", invoke_args)

    def hypot(self, right: Union[engine.Number, int, int, float]) -> engine.Number:
        if right is not None and not isinstance(
            right, (engine.Number, int, int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持(engine.Number,int,int,float)类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "input": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Number.hypot", "engine.Number", invoke_args)

    def max(self, right: Union[engine.Number, int, int, float]) -> engine.Number:
        if right is not None and not isinstance(
            right, (engine.Number, int, int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持(engine.Number,int,int,float)类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "input": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Number.max", "engine.Number", invoke_args)

    def min(self, right: Union[engine.Number, int, int, float]) -> engine.Number:
        if right is not None and not isinstance(
            right, (engine.Number, int, int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持(engine.Number,int,int,float)类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "input": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Number.min", "engine.Number", invoke_args)

    def multiply(self, right: Union[engine.Number, int, int, float]) -> engine.Number:
        if right is not None and not isinstance(
            right, (engine.Number, int, int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持(engine.Number,int,int,float)类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "input": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Number.multiply", "engine.Number", invoke_args)

    def pow(self, right: Union[engine.Number, int, int, float]) -> engine.Number:
        if right is not None and not isinstance(
            right, (engine.Number, int, int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持(engine.Number,int,int,float)类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "input": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Number.pow", "engine.Number", invoke_args)

    def add(self, right: Union[engine.Number, int, int, float]) -> engine.Number:
        if right is not None and not isinstance(
            right, (engine.Number, int, int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持(engine.Number,int,int,float)类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "input": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Number.add", "engine.Number", invoke_args)

    def subtract(self, right: Union[engine.Number, int, int, float]) -> engine.Number:
        if right is not None and not isinstance(
            right, (engine.Number, int, int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持(engine.Number,int,int,float)类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "input": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Number.subtract", "engine.Number", invoke_args)

    def divide(self, right: Union[engine.Number, int, int, float]) -> engine.Number:
        if right is not None and not isinstance(
            right, (engine.Number, int, int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持(engine.Number,int,int,float)类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "input": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Number.divide", "engine.Number", invoke_args)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .task import Task
from .maps import Maps
from aiearth.core.client.endpoints import Endpoints
from .interactive_session import InteractiveSession
from .client import BaseClient, Client
from .dataset import Dataset



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

import abc
import inspect
from typing import Union
from types import FunctionType

from aiearth import engine
from aiearth.engine.variable_node import VariableNode
from aiearth.engine.function_node import FunctionNode
from aiearth.engine.customfunction_node import CustomFunctionNode
from aiearth.engine.function_helper import FunctionHelper
from aiearth.core.error.aie_error import AIEError, AIEErrorCode


class Model(FunctionNode):
    def __init__(self, modelName) -> engine.Model:
        self.modelName = modelName
        super(Model, self).__init__("Model.load", {"modelName": modelName})

    def predict(self, src: engine.Image, dst: engine.Image = None) -> engine.Image:
        if src is not None and not isinstance(src, engine.Image):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"src 只支持engine.Image类型参数, 传入类型为{type(src)}"
            )

        if dst is not None and not isinstance(dst, engine.Image):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"dst 只支持engine.Image类型参数, 传入类型为{type(dst)}"
            )

        invoke_args = {
            "model": self,
            "src": src,
            "dst": dst,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "src" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数src不能为空")

        return FunctionHelper.apply("Model.predict", "engine.Image", invoke_args)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .gen_visitor import GenVisitor
import json
from .optimizer import Optimizer


class Serializer(object):
    def __init__(self):
        pass

    def encode(self, obj, is_compacted=True) -> str:
        visitor = GenVisitor(is_compacted)
        ref = visitor.visit(obj)
        optimizer = Optimizer(ref, visitor.scope, visitor.ref_uses_cnt)
        optimize_ref = optimizer.optimize()
        result = {"result": optimize_ref, "values": optimizer.optimized_values}

        # visitor = GenVisitor(False)
        # value = visitor.visit(obj)
        # result = {"result": 0, "values": {"0": value}}

        return json.dumps(result, ensure_ascii=False, allow_nan=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
from aiearth.core.error import AIEError, AIEErrorCode
from aiearth.core.client.client import BaseClient

from aiearth.engine import aie_env

import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class Client(BaseClient):
    class ResponseCode(object):
        OK = 0
        ERROR = -1

    @staticmethod
    def __append_extra_body(data):
        options = {}
        if "options" in data:
            options = json.loads(data["options"])

        options["sessionId"] = aie_env.AIEEnv.getCurrentUserInteractiveSession()
        options["projectId"] = aie_env.AIEEnv.getCurrentUserProjectId()
        data["options"] = json.dumps(options)
        return data

    @staticmethod
    def post(url, data):
        headers = {"Content-Type": "application/json"}
        data = Client.__append_extra_body(data)
        resp = super(Client, Client).post(url, headers, data)
        response_data = resp.json()
        if "object" in response_data:
            value = json.loads(response_data["object"])
            if value["code"] != Client.ResponseCode.OK:
                extrainfo = {}
                if "requestId" in value:
                    extrainfo["requestId"] = value["requestId"]
                raise AIEError(
                    value["code"], value["message"], json.dumps(extrainfo))
            return value
        else:
            raise AIEError(AIEErrorCode.DEFAULT_INTERNAL_ERROR,
                           "", json.dumps(response_data))

    @staticmethod
    def get(url):
        headers = {"Content-Type": "application/json"}
        resp = super(Client, Client).get(url, headers)
        response_data = resp.json()
        if "object" in response_data:
            value = json.loads(response_data["object"])
            if value["code"] != Client.ResponseCode.OK:
                extrainfo = {}
                if "requestId" in value:
                    extrainfo["requestId"] = value["requestId"]
                raise AIEError(
                    value["code"], value["message"], json.dumps(extrainfo))
            return value
        else:
            raise AIEError(AIEErrorCode.DEFAULT_INTERNAL_ERROR,
                           "", json.dumps(response_data))

    @staticmethod
    def delete(url):
        headers = {"Content-Type": "application/json"}
        resp = super(Client, Client).delete(url, headers)
        response_data = resp.json()
        if "object" in response_data:
            value = json.loads(response_data["object"])
            if value["code"] != Client.ResponseCode.OK:
                extrainfo = {}
                if "requestId" in value:
                    extrainfo["requestId"] = value["requestId"]
                raise AIEError(
                    value["code"], value["message"], json.dumps(extrainfo))
            return value
        else:
            raise AIEError(AIEErrorCode.DEFAULT_INTERNAL_ERROR,
                           "", json.dumps(response_data))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from abc import ABCMeta, abstractmethod

from aiearth.engine.udf_base.images import ImageSet, Image
from aiearth.engine.udf_base.udf_utils import UDFUtils


class AIEUDF(metaclass=ABCMeta):

    @abstractmethod
    def execute(self, image_set: ImageSet) -> Image:
        pass

    def do_execute(self, images_wrapper) -> bytes:
        image_set = UDFUtils.images_wrapper_to_set(images_wrapper)
        result_image = self.execute(image_set)
        return UDFUtils.image_to_wrapper(result_image).SerializeToString()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from enum import Enum
from typing import List, Dict


class DataType(Enum):
    UINT8 = 'uint8'
    INT8 = 'int8'
    INT16 = 'int16'
    UINT16 = 'uint16'
    INT32 = 'int32'
    FLOAT32 = 'float32'
    FLOAT64 = 'float64'

    @staticmethod
    def get_type_by_value(value: str):
        for dataType in DataType:
            if dataType.value == value:
                return dataType


class Band(object):
    def __init__(self, data_array, data_type, nodata_value, cols, rows):
        self.data_array = data_array
        self.data_type = data_type
        self.nodata_value = nodata_value
        self.cols = cols
        self.rows = rows


class Image(object):
    def __init__(self, bands: Dict[str, Band]):
        self.bands = bands


class ImageSet(object):
    def __init__(self, images: List[Image], params):
        self.images = images
        self.params = params



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from aiearth import engine
from aiearth.core import g_var
from aiearth.engine.client.interactive_session import InteractiveSession
from aiearth.core.error import AIEError, AIEErrorCode


class AIEEnv(object):
    @staticmethod
    def init(debug_level, enable_udf=False):
        AIEEnv.__setDebugLevel(debug_level)

        InteractiveSession.create(enable_udf=enable_udf)
        AIEEnv.__setPorjectId()

        print("计算资源初始化完成.")

    @staticmethod
    def cancelAllInteractiveJobs():
        confirm = input("该操作会停止当前账号所有正在运行中的在线交互任务，请输入yes/no: ")
        if confirm == "yes":
            print("在线交互任务取消中...")
            aie.client.interactive_session.InteractiveSession.cancelAllJobs()
            print("操作完成")
        else:
            print("操作取消")

    @staticmethod
    def __setPorjectId():
        if os.getenv(g_var.JupyterEnv.PROJECT_ID) is not None:
            project_id = os.getenv(g_var.JupyterEnv.PROJECT_ID)
            if project_id.isdigit():
                g_var.set_var(g_var.GVarKey.Project.PROJECT_ID,
                              int(project_id))
            else:
                print("warning: 项目ID获取失败，设置为-1")
                g_var.set_var(g_var.GVarKey.Project.PROJECT_ID, -1)
        else:
            g_var.set_var(g_var.GVarKey.Project.PROJECT_ID, -1)

    @staticmethod
    def __setDebugLevel(debug_level):
        g_var.set_var(g_var.GVarKey.Log.LOG_LEVEL, debug_level)

    @staticmethod
    def getCurrentUserInteractiveSession():
        if not g_var.has_var(g_var.GVarKey.InteractiveSession.INTERACTIVE_SESSION_ID):
            raise AIEError(AIEErrorCode.ENVIRONMENT_INIT_ERROR,
                           "计算资源ID获取失败，请先调用 aie.Initialize() 创建")
        return g_var.get_var(g_var.GVarKey.InteractiveSession.INTERACTIVE_SESSION_ID)

    @staticmethod
    def getCurrentUserProjectId():
        if not g_var.has_var(g_var.GVarKey.Project.PROJECT_ID):
            raise AIEError(AIEErrorCode.ENVIRONMENT_INIT_ERROR,
                           "项目projectId获取失败")
        return g_var.get_var(g_var.GVarKey.Project.PROJECT_ID)

    @staticmethod
    def getDebugLevel():
        return g_var.get_var(g_var.GVarKey.Log.LOG_LEVEL)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

import abc
import inspect
from typing import Union
from types import FunctionType

from aiearth import engine
from aiearth.engine.variable_node import VariableNode
from aiearth.engine.function_node import FunctionNode
from aiearth.engine.customfunction_node import CustomFunctionNode
from aiearth.engine.function_helper import FunctionHelper
from aiearth.core.error.aie_error import AIEError, AIEErrorCode


class Feature(FunctionNode):
    def __init__(self, geometry, properties=None) -> engine.Feature:
        args = geometry
        if isinstance(args, engine.Geometry):
            if properties is not None and not isinstance(
                properties, (dict, engine.function_node.FunctionNode)
            ):
                raise AIEError(
                    AIEErrorCode.ARGS_ERROR,
                    f"properties 只支持dict|engine.function_node.FunctionNode类型参数, 传入类型为{type(properties)}",
                )

            invoke_args = {
                "geometry": geometry,
                "properties": properties,
            }

            invoke_args = {k: v for k, v in invoke_args.items() if v is not None}
            super(Feature, self).__init__("Feature", invoke_args)
        elif isinstance(args, engine.variable_node.VariableNode):
            super(Feature, self).__init__(
                args.func_name, args.invoke_args, args.var_name
            )
        elif isinstance(args, engine.function_node.FunctionNode):
            super(Feature, self).__init__(
                args.func_name, args.invoke_args, args.var_name
            )
        elif args is None:
            invoke_args = {
                "geometry": geometry,
                "properties": properties,
            }

            invoke_args = {k: v for k, v in invoke_args.items() if v is not None}
            super(Feature, self).__init__("Feature", invoke_args)
        else:
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"geometry 只支持engine.Geometry类型参数, 传入类型为{type(geometry)}",
            )

    def getMapId(self, vis_params):
        if vis_params is not None and not isinstance(vis_params, dict):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"vis_params 只支持dict类型参数, 传入类型为{type(vis_params)}",
            )
        return engine.client.Maps.getMapId(self, vis_params)

    def getCenter(self) -> tuple:
        bbox = engine.client.InteractiveSession.getBounds(self)
        if bbox is not None and isinstance(bbox, list):
            center = ((bbox[0] + bbox[2]) / 2, (bbox[1] + bbox[3]) / 2)
            return center
        raise AIEError(AIEErrorCode.ARGS_ERROR, f"获取Center失败. bbox: {bbox}")

    def getBounds(self) -> list:
        bbox = engine.client.InteractiveSession.getBounds(self)
        if bbox is not None and isinstance(bbox, list):
            bounds = [bbox[0], bbox[1], bbox[2], bbox[3]]
            return bounds
        raise AIEError(AIEErrorCode.ARGS_ERROR, f"获取Bounds失败. bbox: {bbox}")

    def get(self, property: str) -> object:
        if property is not None and not isinstance(property, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"property 只支持str类型参数, 传入类型为{type(property)}"
            )

        invoke_args = {
            "input": self,
            "property": property,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "property" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数property不能为空")

        return FunctionHelper.apply("Feature.get", "object", invoke_args)

    def propertyNames(self) -> object:
        invoke_args = {
            "element": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Feature.propertyNames", "object", invoke_args)

    def select(
        self,
        propertySelectors: list,
        newProperties: list = None,
        retainGeometry: bool = True,
    ) -> object:
        if propertySelectors is not None and not isinstance(propertySelectors, list):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"propertySelectors 只支持list类型参数, 传入类型为{type(propertySelectors)}",
            )

        if newProperties is not None and not isinstance(newProperties, list):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"newProperties 只支持list类型参数, 传入类型为{type(newProperties)}",
            )

        if retainGeometry is not None and not isinstance(retainGeometry, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"retainGeometry 只支持bool类型参数, 传入类型为{type(retainGeometry)}",
            )

        invoke_args = {
            "input": self,
            "propertySelectors": propertySelectors,
            "newProperties": newProperties,
            "retainGeometry": retainGeometry,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "propertySelectors" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数propertySelectors不能为空")

        return FunctionHelper.apply("Feature.select", "object", invoke_args)

    def toArray(self, properties: list) -> object:
        if properties is not None and not isinstance(properties, list):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"properties 只支持list类型参数, 传入类型为{type(properties)}",
            )

        invoke_args = {
            "feature": self,
            "properties": properties,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "properties" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数properties不能为空")

        return FunctionHelper.apply("Feature.toArray", "object", invoke_args)

    def toDictionary(self, properties: list = None) -> object:
        if properties is not None and not isinstance(properties, list):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"properties 只支持list类型参数, 传入类型为{type(properties)}",
            )

        invoke_args = {
            "element": self,
            "properties": properties,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Feature.toDictionary", "object", invoke_args)

    def set(self, var_args: object) -> engine.Feature:
        if var_args is not None and not isinstance(var_args, object):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"var_args 只支持object类型参数, 传入类型为{type(var_args)}",
            )

        invoke_args = {
            "element": self,
            "var_args": var_args,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "var_args" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数var_args不能为空")

        return FunctionHelper.apply("Feature.set", "engine.Feature", invoke_args)

    def setGeometry(self, geometry: engine.Geometry) -> engine.Feature:
        if geometry is not None and not isinstance(geometry, engine.Geometry):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"geometry 只支持engine.Geometry类型参数, 传入类型为{type(geometry)}",
            )

        invoke_args = {
            "feature": self,
            "geometry": geometry,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "geometry" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数geometry不能为空")

        return FunctionHelper.apply(
            "Feature.setGeometry", "engine.Feature", invoke_args
        )

    def area(self) -> object:
        invoke_args = {
            "feature": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Feature.area", "object", invoke_args)

    def perimeter(self) -> object:
        invoke_args = {
            "feature": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Feature.perimeter", "object", invoke_args)

    def length(self) -> object:
        invoke_args = {
            "feature": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Feature.length", "object", invoke_args)

    def bounds(self) -> engine.Feature:
        invoke_args = {
            "feature": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Feature.bounds", "engine.Feature", invoke_args)

    def centroid(self) -> engine.Feature:
        invoke_args = {
            "feature": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Feature.centroid", "engine.Feature", invoke_args)

    def convexHull(self) -> engine.Feature:
        invoke_args = {
            "feature": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Feature.convexHull", "engine.Feature", invoke_args)

    def cutLines(self, distances: list) -> engine.Feature:
        if distances is not None and not isinstance(distances, list):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"distances 只支持list类型参数, 传入类型为{type(distances)}",
            )

        invoke_args = {
            "feature": self,
            "distances": distances,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "distances" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数distances不能为空")

        return FunctionHelper.apply("Feature.cutLines", "engine.Feature", invoke_args)

    def simplify(self, tolerance: [int, float]) -> engine.Feature:
        if tolerance is not None and not isinstance(tolerance, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"tolerance 只支持(int,float)类型参数, 传入类型为{type(tolerance)}",
            )

        invoke_args = {
            "feature": self,
            "tolerance": tolerance,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "tolerance" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数tolerance不能为空")

        return FunctionHelper.apply("Feature.simplify", "engine.Feature", invoke_args)

    def transform(self, proj: str) -> engine.Feature:
        if proj is not None and not isinstance(proj, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"proj 只支持str类型参数, 传入类型为{type(proj)}"
            )

        invoke_args = {
            "feature": self,
            "proj": proj,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "proj" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数proj不能为空")

        return FunctionHelper.apply("Feature.transform", "engine.Feature", invoke_args)

    def containedIn(self, right: engine.Feature) -> object:
        if right is not None and not isinstance(right, engine.Feature):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持engine.Feature类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "left": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Feature.containedIn", "object", invoke_args)

    def contains(self, right: engine.Feature) -> object:
        if right is not None and not isinstance(right, engine.Feature):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持engine.Feature类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "left": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Feature.contains", "object", invoke_args)

    def difference(self, right: engine.Feature) -> engine.Feature:
        if right is not None and not isinstance(right, engine.Feature):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持engine.Feature类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "left": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Feature.difference", "engine.Feature", invoke_args)

    def disjoint(self, right: engine.Feature) -> object:
        if right is not None and not isinstance(right, engine.Feature):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持engine.Feature类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "left": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Feature.disjoint", "object", invoke_args)

    def dissolve(self) -> engine.Feature:
        invoke_args = {
            "feature": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Feature.dissolve", "engine.Feature", invoke_args)

    def distance(self, right: engine.Feature) -> object:
        if right is not None and not isinstance(right, engine.Feature):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持engine.Feature类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "left": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Feature.distance", "object", invoke_args)

    def withinDistance(self, right: engine.Feature, distance: [int, float]) -> object:
        if right is not None and not isinstance(right, engine.Feature):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持engine.Feature类型参数, 传入类型为{type(right)}",
            )

        if distance is not None and not isinstance(distance, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"distance 只支持(int,float)类型参数, 传入类型为{type(distance)}",
            )

        invoke_args = {
            "left": self,
            "right": right,
            "distance": distance,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        if "distance" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数distance不能为空")

        return FunctionHelper.apply("Feature.withinDistance", "object", invoke_args)

    def intersection(self, right: engine.Feature) -> engine.Feature:
        if right is not None and not isinstance(right, engine.Feature):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持engine.Feature类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "left": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply(
            "Feature.intersection", "engine.Feature", invoke_args
        )

    def intersects(self, right: engine.Feature) -> object:
        if right is not None and not isinstance(right, engine.Feature):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持engine.Feature类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "left": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Feature.intersects", "object", invoke_args)

    def symmetricDifference(self, right: engine.Feature) -> engine.Feature:
        if right is not None and not isinstance(right, engine.Feature):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持engine.Feature类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "left": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply(
            "Feature.symmetricDifference", "engine.Feature", invoke_args
        )

    def union(self, right: engine.Feature) -> engine.Feature:
        if right is not None and not isinstance(right, engine.Feature):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"right 只支持engine.Feature类型参数, 传入类型为{type(right)}",
            )

        invoke_args = {
            "left": self,
            "right": right,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "right" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数right不能为空")

        return FunctionHelper.apply("Feature.union", "engine.Feature", invoke_args)

    def buffer(self, distance: [int, float]) -> engine.Feature:
        if distance is not None and not isinstance(distance, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"distance 只支持(int,float)类型参数, 传入类型为{type(distance)}",
            )

        invoke_args = {
            "feature": self,
            "distance": distance,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "distance" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数distance不能为空")

        return FunctionHelper.apply("Feature.buffer", "engine.Feature", invoke_args)

    def copyProperties(
        self, source: engine.Feature, properties: list = None, exclude: list = None
    ) -> engine.Feature:
        if source is not None and not isinstance(source, engine.Feature):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"source 只支持engine.Feature类型参数, 传入类型为{type(source)}",
            )

        if properties is not None and not isinstance(properties, list):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"properties 只支持list类型参数, 传入类型为{type(properties)}",
            )

        if exclude is not None and not isinstance(exclude, list):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"exclude 只支持list类型参数, 传入类型为{type(exclude)}"
            )

        invoke_args = {
            "destination": self,
            "source": source,
            "properties": properties,
            "exclude": exclude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "source" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数source不能为空")

        return FunctionHelper.apply(
            "Feature.copyProperties", "engine.Feature", invoke_args
        )

    def voronoiDiagram(self) -> engine.Feature:
        invoke_args = {
            "feature": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply(
            "Feature.voronoiDiagram", "engine.Feature", invoke_args
        )

    def delaunayTriangulation(self) -> engine.Feature:
        invoke_args = {
            "feature": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply(
            "Feature.delaunayTriangulation", "engine.Feature", invoke_args
        )

    def smooth(
        self, iterations: int, offset: [int, float], maxAngle: [int, float]
    ) -> engine.Feature:
        if iterations is not None and not isinstance(iterations, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"iterations 只支持int类型参数, 传入类型为{type(iterations)}",
            )

        if offset is not None and not isinstance(offset, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"offset 只支持(int,float)类型参数, 传入类型为{type(offset)}",
            )

        if maxAngle is not None and not isinstance(maxAngle, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"maxAngle 只支持(int,float)类型参数, 传入类型为{type(maxAngle)}",
            )

        invoke_args = {
            "feature": self,
            "iterations": iterations,
            "offset": offset,
            "maxAngle": maxAngle,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "iterations" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数iterations不能为空")

        if "offset" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数offset不能为空")

        if "maxAngle" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数maxAngle不能为空")

        return FunctionHelper.apply("Feature.smooth", "engine.Feature", invoke_args)

    def getNumber(self, property: str) -> object:
        if property is not None and not isinstance(property, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"property 只支持str类型参数, 传入类型为{type(property)}"
            )

        invoke_args = {
            "input": self,
            "property": property,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "property" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数property不能为空")

        return FunctionHelper.apply("Feature.getNumber", "object", invoke_args)

    def getString(self, property: str) -> object:
        if property is not None and not isinstance(property, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"property 只支持str类型参数, 传入类型为{type(property)}"
            )

        invoke_args = {
            "input": self,
            "property": property,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "property" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数property不能为空")

        return FunctionHelper.apply("Feature.getString", "object", invoke_args)

    def renamePropertyNames(self, var_args: dict) -> engine.Feature:
        if var_args is not None and not isinstance(var_args, dict):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"var_args 只支持dict类型参数, 传入类型为{type(var_args)}"
            )

        invoke_args = {
            "input": self,
            "var_args": var_args,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "var_args" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数var_args不能为空")

        return FunctionHelper.apply(
            "Feature.renamePropertyNames", "engine.Feature", invoke_args
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import collections.abc
from typing import Iterator

from aiearth import engine
import ipyleaflet
from ipyleaflet import projections, basemaps
from .aie_layer import AIEImageLayer, AIEGeometryLayer
from aiearth.core.error import AIEError, AIEErrorCode

# init the aie_color module when aie_map module is initialized
from aiearth.engine.map import aie_color
aie_color.init()


def _layer_name_generator(i: int = 1) -> Iterator:
    while True:
        yield "Layer %d" % i
        i += 1


class Map(ipyleaflet.Map):
    def __init__(self, **kwargs):
        self._layer_name_iterator = _layer_name_generator()
        # crs
        if "crs" in kwargs.keys():
            kwargs.pop("crs")
            print("warning. 暂不支持坐标系设置，默认为EPSG4326")
        # center
        if "location" in kwargs.keys():
            kwargs["center"] = kwargs["location"]
            kwargs.pop("location")
        if "center" not in kwargs.keys():
            # Beijing
            kwargs["center"] = (116.383331, 39.916668)
        else: 
            center = kwargs["center"]
            kwargs["center"] = (center[1], center[0])
        # zoom
        if "zoom_start" in kwargs.keys():
            kwargs["zoom"] = kwargs["zoom_start"]
            kwargs.pop("zoom_start")
        if "zoom" not in kwargs.keys():
            kwargs["zoom"] = 2
        if "max_zoom" not in kwargs.keys():
            kwargs["max_zoom"] = 18
        # attribution_control
        if "attribution_control" not in kwargs.keys():
            kwargs["attribution_control"] = False
        # style
        if "height" not in kwargs.keys():
            self._height = "600px"
        else:
            if isinstance(kwargs["height"], int):
                self._height = str(kwargs["height"]) + "px"
            else:
                self._height = kwargs["height"]
            kwargs.pop("height")
        if "width" not in kwargs.keys():
            self._width = "100%"
        else:
            if isinstance(kwargs["width"], int):
                self._width = str(kwargs["width"]) + "px"
            else:
                self._width = kwargs["width"]
            kwargs.pop("width")
        # controls
        lite_mode = False
        if "lite_mode" in kwargs.keys():
            lite_mode = kwargs["lite_mode"]
            kwargs.pop("lite_mode")
        self._has_layers_control = (
            kwargs["layers_control"]
            if "layers_control" in kwargs.keys()
            else not lite_mode
        )
        self._has_scale_ctrl = (
            kwargs["scale_ctrl"] if "scale_ctrl" in kwargs.keys() else not lite_mode
        )
        self._has_fullscreen_ctrl = (
            kwargs["fullscreen_ctrl"]
            if "fullscreen_ctrl" in kwargs.keys()
            else not lite_mode
        )
        self._has_measure_ctrl = (
            kwargs["measure_ctrl"] if "measure_ctrl" in kwargs.keys(
            ) else not lite_mode
        )

        super().__init__(**kwargs)

        self.layout.height = self._height
        self.layout.width = self._width
        if self._has_layers_control:
            self.add_control(ipyleaflet.LayersControl(position="topright"))
        if self._has_scale_ctrl:
            self.add_control(ipyleaflet.ScaleControl(position="bottomleft"))
        if self._has_fullscreen_ctrl:
            self.add_control(ipyleaflet.FullScreenControl())
        if self._has_measure_ctrl:
            self.add_control(
                ipyleaflet.MeasureControl(
                    position="bottomleft",
                    active_color="orange",
                    primary_length_unit="kilometers",
                )
            )

    def add_aie_layer(
        self, aie_object, vis_params, name=None, show=True, opacity=1.0, **kwargs
    ):
        # # try to translate color palette
        # if isinstance(vis_params, dict) \
        #     and 'palette' in vis_params.keys() \
        #         and isinstance(vis_params['palette'], collections.abc.Iterable):
        #     vis_params['palette'] = [aie_color.translate_color(color) for color in vis_params['palette']]

        if name is None:
            name = self._layer_name_iterator.__next__()
        kwargs.update(
            {
                "visible": show,
                "opacity": opacity,
                "format": "image/png",
                "transparent": True,
            }
        )
        if isinstance(aie_object, (engine.Image, engine.ImageCollection)):
            self.add_layer(AIEImageLayer(
                name, aie_object, vis_params, **kwargs))
        elif isinstance(aie_object, (engine.Geometry, engine.Feature, engine.FeatureCollection)):
            self.add_layer(AIEGeometryLayer(
                name, aie_object, vis_params, **kwargs))
        else:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "对于类型 %s 的 layer 未实现" % type(aie_object))

    projections = projections
    addLayer = add_aie_layer



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
from typing import List
import ast

import astunparse as astunparse
from aiearth.core.error import AIEError, AIEErrorCode


class InputParam(object):
    def __init__(self, band_names: List[str]):
        self.input_band_names = band_names

    def to_json(self):
        return json.dumps(self, default=lambda o: o.__dict__, indent=4)


class OutputParam(object):
    def __init__(self, band_names: List[str]):
        self.output_band_names = band_names

    def to_json(self):
        return json.dumps(self, default=lambda o: o.__dict__, indent=4)


def aie_raster_local_udf(input_bands: List[InputParam], output_bands: OutputParam):
    def decorator(cls):
        def wrapper(*args, **kwargs):
            instance = cls(*args, **kwargs)
            return instance

        setattr(wrapper, "udf", "true")
        setattr(wrapper, "clz", cls)
        return wrapper

    return decorator


def aie_raster_zonal_udaf(input_bands: List[InputParam]):
    def decorator(cls):
        def wrapper(*args, **kwargs):
            instance = cls(*args, **kwargs)
            return instance

        setattr(wrapper, "udaf", "true")
        setattr(wrapper, "clz", cls)
        return wrapper

    return decorator


def annotation_parser(source_code):
    module = ast.parse(source_code)
    # 找到使用@aie_raster_local_udf装饰的类
    class_name = None
    target_class = None
    for node in ast.walk(module):
        if isinstance(node, ast.ClassDef):
            for decorator in node.decorator_list:
                if isinstance(decorator, ast.Call) and (decorator.func.id == 'aie_raster_local_udf' or decorator.func.id == 'aie_raster_zonal_udaf'):
                    target_class = node
                    break
            if target_class:
                class_name = target_class.name
                break
    if class_name is None or target_class is None:
        raise AIEError(AIEErrorCode.DEFAULT_INTERNAL_ERROR, "注解解析失败，无法获取class_name")

    input_bands = []
    output_bands = []
    for decorator in target_class.decorator_list:
        if isinstance(decorator, ast.Call) and (decorator.func.id == 'aie_raster_local_udf' or decorator.func.id == 'aie_raster_zonal_udaf'):
            for keyword in decorator.keywords:
                if keyword.arg == 'input_bands':
                    for item in eval(astunparse.unparse(keyword.value)):
                        input_bands.append(item.input_band_names)
                elif keyword.arg == 'output_bands':
                    output_bands = eval(astunparse.unparse(keyword.value)).output_band_names

    meta_map = {**InputParam(input_bands).__dict__, **OutputParam(output_bands).__dict__, **{'class_name': class_name}}
    if 'input_band_names' not in meta_map or 'output_band_names' not in meta_map or 'class_name' not in meta_map:
        raise AIEError(AIEErrorCode.DEFAULT_INTERNAL_ERROR, f"注解解析失败，无法获取input_band_names|output_band_names|class_name; meta={json.dumps(meta_map, separators=(',', ':'))}")
    return json.dumps(meta_map, separators=(',', ':'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
from aiearth.core.client.endpoints import *
from .client import Client
from aiearth.engine.serialize import serializer
from aiearth.core.error import AIEError, AIEErrorCode


class Task(object):
    class StatusCode(object):
        WAITING = "waiting"
        RUNNING = "running"
        COMPLETED = "completed"
        FAILED = "failed"
        CANCEL_REQUEST = "cancel_request"
        CANCELLED = "cancelled"

    class Status(object):
        WAITING = "等待中"
        RUNNING = "运行中"
        COMPLETED = "任务已完成"
        FAILED = "任务失败"
        CANCEL_REQUEST = "任务取消中"
        CANCELLED = "任务已取消"

    def __init__(self, obj, options=None):
        self.request_id = None
        self.expr = serializer.encode(obj)
        self.options = options

    def start(self):
        self.request_id = newUUID()
        data = {"requestId": self.request_id,
                "expr": self.expr}

        if self.options is not None:
            data["options"] = json.dumps(self.options)

        headers = {"Content-Type": "application/json"}
        url = Endpoints.SDK_GATEWAY + SdkGatewayResource.Task.START
        Client.post(url, data)

    def __toStatus(self, status):
        if status == Task.StatusCode.WAITING:
            return Task.Status.WAITING
        elif status == Task.StatusCode.RUNNING:
            return Task.Status.RUNNING
        elif status == Task.StatusCode.COMPLETED:
            return Task.Status.COMPLETED
        elif status == Task.StatusCode.FAILED:
            return Task.Status.FAILED
        elif status == Task.StatusCode.CANCEL_REQUEST:
            return Task.Status.CANCEL_REQUEST
        elif status == Task.StatusCode.CANCELLED:
            return Task.Status.CANCELLED
        else:
            raise AIEError(AIEErrorCode.DEFAULT_INTERNAL_ERROR,
                           "", f"status {status} 未知")

    def status(self):
        headers = {"Content-Type": "application/json"}
        url = Endpoints.SDK_GATEWAY + \
            SdkGatewayResource.Task.STATUS.format(self.request_id)
        response = Client.get(url)
        status = response["status"]
        return self.__toStatus(status)

    def active(self):
        status = self.status()
        return status in (Task.Status.WAITING, Task.Status.RUNNING)

    def cancel(self):
        headers = {"Content-Type": "application/json"}
        url = Endpoints.SDK_GATEWAY + \
            SdkGatewayResource.Task.CANCEL.format(self.request_id)
        Client.delete(url)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__version__ = "3.1.3"

import aiearth.core

from aiearth.engine.aie_env import AIEEnv
from aiearth.core import g_var
from aiearth.engine import serialize
from aiearth.core import auth
from aiearth.engine import client
from aiearth.core import error
from aiearth.engine.map.aie_map import Map


from aiearth.engine.aie_object.image import Image

from aiearth.engine.aie_object.collection import Collection

from aiearth.engine.aie_object.image_collection import ImageCollection

from aiearth.engine.aie_object.geometry import Geometry

from aiearth.engine.aie_object.feature import Feature

from aiearth.engine.aie_object.feature_collection import FeatureCollection

from aiearth.engine.aie_object.filter import Filter

from aiearth.engine.aie_object.reducer import Reducer

from aiearth.engine.aie_object.terrain import Terrain

from aiearth.engine.aie_object.kernel import Kernel

from aiearth.engine.aie_object.classifier import Classifier

from aiearth.engine.aie_object.confusion_matrix import ConfusionMatrix

from aiearth.engine.aie_object.model import Model

from aiearth.engine.aie_object.list import List

from aiearth.engine.aie_object.projection import Projection

from aiearth.engine.aie_object.number import Number


Authenticate = aiearth.core.Authenticate


def Initialize(debug_level=g_var.LogLevel.INFO_LEVEL, debug=False, enable_udf=False):
    if debug:
        debug_level = g_var.LogLevel.DEBUG_LEVEL
    AIEEnv.init(debug_level, enable_udf=enable_udf)


def se(obj):
    return serialize.serializer.encode(obj)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

import abc
import inspect
from typing import Union
from types import FunctionType

from aiearth import engine
from aiearth.engine.variable_node import VariableNode
from aiearth.engine.function_node import FunctionNode
from aiearth.engine.customfunction_node import CustomFunctionNode
from aiearth.engine.function_helper import FunctionHelper
from aiearth.core.error.aie_error import AIEError, AIEErrorCode


class Classifier(FunctionNode):
    def __init__(self, preResult: str = "") -> engine.Classifier:
        if preResult is not None and not isinstance(preResult, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"preResult 只支持str类型参数, 传入类型为{type(preResult)}"
            )

        invoke_args = {
            "preResult": preResult,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        super(Classifier, self).__init__("Classifier.constructors", invoke_args)

    def train(
        self,
        features: engine.FeatureCollection,
        classProperty: str,
        inputProperties: list,
    ) -> engine.Classifier:
        if features is not None and not isinstance(features, engine.FeatureCollection):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"features 只支持engine.FeatureCollection类型参数, 传入类型为{type(features)}",
            )

        if classProperty is not None and not isinstance(classProperty, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"classProperty 只支持str类型参数, 传入类型为{type(classProperty)}",
            )

        if inputProperties is not None and not isinstance(inputProperties, list):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"inputProperties 只支持list类型参数, 传入类型为{type(inputProperties)}",
            )

        invoke_args = {
            "input": self,
            "features": features,
            "classProperty": classProperty,
            "inputProperties": inputProperties,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "features" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数features不能为空")

        if "classProperty" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数classProperty不能为空")

        if "inputProperties" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数inputProperties不能为空")

        clf = FunctionHelper.apply("Classifier.train", "engine.Classifier", invoke_args)

        obj = engine.client.InteractiveSession.classifierTrain(clf)
        invoke_args["preResult"] = obj
        clf = FunctionHelper.apply("Classifier.train", "engine.Classifier", invoke_args)

        return clf

    @staticmethod
    def linearsvm() -> engine.Classifier:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply(
            "Classifier.linearsvm", "engine.Classifier", invoke_args
        )

    @staticmethod
    def libsvm(
        svmType: str = "C_SVC",
        kernelType: str = "LINEAR",
        shrinking: bool = True,
        degree: int = 3,
        gamma: [int, float] = None,
        coef0: [int, float] = 0.0,
        cost: [int, float] = 1.0,
        nu: [int, float] = 0.5,
        terminationEpsilon: [int, float] = 0.001,
        lossEpsilon: [int, float] = 0.1,
    ) -> engine.Classifier:
        if svmType is not None and not isinstance(svmType, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"svmType 只支持str类型参数, 传入类型为{type(svmType)}"
            )

        if kernelType is not None and not isinstance(kernelType, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"kernelType 只支持str类型参数, 传入类型为{type(kernelType)}",
            )

        if shrinking is not None and not isinstance(shrinking, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"shrinking 只支持bool类型参数, 传入类型为{type(shrinking)}",
            )

        if degree is not None and not isinstance(degree, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"degree 只支持int类型参数, 传入类型为{type(degree)}"
            )

        if gamma is not None and not isinstance(gamma, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"gamma 只支持(int,float)类型参数, 传入类型为{type(gamma)}"
            )

        if coef0 is not None and not isinstance(coef0, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"coef0 只支持(int,float)类型参数, 传入类型为{type(coef0)}"
            )

        if cost is not None and not isinstance(cost, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"cost 只支持(int,float)类型参数, 传入类型为{type(cost)}"
            )

        if nu is not None and not isinstance(nu, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"nu 只支持(int,float)类型参数, 传入类型为{type(nu)}"
            )

        if terminationEpsilon is not None and not isinstance(
            terminationEpsilon, (int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"terminationEpsilon 只支持(int,float)类型参数, 传入类型为{type(terminationEpsilon)}",
            )

        if lossEpsilon is not None and not isinstance(lossEpsilon, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"lossEpsilon 只支持(int,float)类型参数, 传入类型为{type(lossEpsilon)}",
            )

        invoke_args = {
            "svmType": svmType,
            "kernelType": kernelType,
            "shrinking": shrinking,
            "degree": degree,
            "gamma": gamma,
            "coef0": coef0,
            "cost": cost,
            "nu": nu,
            "terminationEpsilon": terminationEpsilon,
            "lossEpsilon": lossEpsilon,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply(
            "Classifier.libsvm", "engine.Classifier", invoke_args
        )

    @staticmethod
    def decisionTree(
        maxDepth: int = 5,
        maxNodes: int = None,
        minLeafPopulation: int = 5,
        minWeightFraction: [int, float] = 0.0,
        minInfoGain: [int, float] = 0.0,
        seed: int = 0,
    ) -> engine.Classifier:
        if maxDepth is not None and not isinstance(maxDepth, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"maxDepth 只支持int类型参数, 传入类型为{type(maxDepth)}"
            )

        if maxNodes is not None and not isinstance(maxNodes, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"maxNodes 只支持int类型参数, 传入类型为{type(maxNodes)}"
            )

        if minLeafPopulation is not None and not isinstance(minLeafPopulation, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"minLeafPopulation 只支持int类型参数, 传入类型为{type(minLeafPopulation)}",
            )

        if minWeightFraction is not None and not isinstance(
            minWeightFraction, (int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"minWeightFraction 只支持(int,float)类型参数, 传入类型为{type(minWeightFraction)}",
            )

        if minInfoGain is not None and not isinstance(minInfoGain, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"minInfoGain 只支持(int,float)类型参数, 传入类型为{type(minInfoGain)}",
            )

        if seed is not None and not isinstance(seed, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"seed 只支持int类型参数, 传入类型为{type(seed)}"
            )

        invoke_args = {
            "maxDepth": maxDepth,
            "maxNodes": maxNodes,
            "minLeafPopulation": minLeafPopulation,
            "minWeightFraction": minWeightFraction,
            "minInfoGain": minInfoGain,
            "seed": seed,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply(
            "Classifier.decisionTree", "engine.Classifier", invoke_args
        )

    @staticmethod
    def cart(
        maxNodes: int = None, maxDepth: int = 20, minLeafPopulation: int = 5
    ) -> engine.Classifier:
        if maxNodes is not None and not isinstance(maxNodes, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"maxNodes 只支持int类型参数, 传入类型为{type(maxNodes)}"
            )

        if maxDepth is not None and not isinstance(maxDepth, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"maxDepth 只支持int类型参数, 传入类型为{type(maxDepth)}"
            )

        if minLeafPopulation is not None and not isinstance(minLeafPopulation, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"minLeafPopulation 只支持int类型参数, 传入类型为{type(minLeafPopulation)}",
            )

        invoke_args = {
            "maxNodes": maxNodes,
            "maxDepth": maxDepth,
            "minLeafPopulation": minLeafPopulation,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Classifier.cart", "engine.Classifier", invoke_args)

    @staticmethod
    def randomForest(
        numTrees: int,
        maxDepth: int = 5,
        minLeafPopulation: int = 5,
        subsamplingRate: [int, float] = 1.0,
        seed: int = 0,
    ) -> engine.Classifier:
        if numTrees is not None and not isinstance(numTrees, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"numTrees 只支持int类型参数, 传入类型为{type(numTrees)}"
            )

        if maxDepth is not None and not isinstance(maxDepth, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"maxDepth 只支持int类型参数, 传入类型为{type(maxDepth)}"
            )

        if minLeafPopulation is not None and not isinstance(minLeafPopulation, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"minLeafPopulation 只支持int类型参数, 传入类型为{type(minLeafPopulation)}",
            )

        if subsamplingRate is not None and not isinstance(
            subsamplingRate, (int, float)
        ):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"subsamplingRate 只支持(int,float)类型参数, 传入类型为{type(subsamplingRate)}",
            )

        if seed is not None and not isinstance(seed, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"seed 只支持int类型参数, 传入类型为{type(seed)}"
            )

        invoke_args = {
            "numTrees": numTrees,
            "maxDepth": maxDepth,
            "minLeafPopulation": minLeafPopulation,
            "subsamplingRate": subsamplingRate,
            "seed": seed,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "numTrees" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数numTrees不能为空")

        return FunctionHelper.apply(
            "Classifier.randomForest", "engine.Classifier", invoke_args
        )

    @staticmethod
    def gradientTreeBoost(
        numTrees: int,
        shrinkage: [int, float] = 0.005,
        samplingRate: [int, float] = 0.7,
        maxDepth: int = 20,
        maxNodes: int = None,
        minLeafPopulation: int = 5,
    ) -> engine.Classifier:
        if numTrees is not None and not isinstance(numTrees, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"numTrees 只支持int类型参数, 传入类型为{type(numTrees)}"
            )

        if shrinkage is not None and not isinstance(shrinkage, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"shrinkage 只支持(int,float)类型参数, 传入类型为{type(shrinkage)}",
            )

        if samplingRate is not None and not isinstance(samplingRate, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"samplingRate 只支持(int,float)类型参数, 传入类型为{type(samplingRate)}",
            )

        if maxDepth is not None and not isinstance(maxDepth, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"maxDepth 只支持int类型参数, 传入类型为{type(maxDepth)}"
            )

        if maxNodes is not None and not isinstance(maxNodes, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"maxNodes 只支持int类型参数, 传入类型为{type(maxNodes)}"
            )

        if minLeafPopulation is not None and not isinstance(minLeafPopulation, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"minLeafPopulation 只支持int类型参数, 传入类型为{type(minLeafPopulation)}",
            )

        invoke_args = {
            "numTrees": numTrees,
            "shrinkage": shrinkage,
            "samplingRate": samplingRate,
            "maxDepth": maxDepth,
            "maxNodes": maxNodes,
            "minLeafPopulation": minLeafPopulation,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "numTrees" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数numTrees不能为空")

        return FunctionHelper.apply(
            "Classifier.gradientTreeBoost", "engine.Classifier", invoke_args
        )

    @staticmethod
    def adaBoost(
        numTrees: int,
        maxDepth: int = 20,
        maxNodes: int = None,
        minLeafPopulation: int = 5,
    ) -> engine.Classifier:
        if numTrees is not None and not isinstance(numTrees, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"numTrees 只支持int类型参数, 传入类型为{type(numTrees)}"
            )

        if maxDepth is not None and not isinstance(maxDepth, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"maxDepth 只支持int类型参数, 传入类型为{type(maxDepth)}"
            )

        if maxNodes is not None and not isinstance(maxNodes, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"maxNodes 只支持int类型参数, 传入类型为{type(maxNodes)}"
            )

        if minLeafPopulation is not None and not isinstance(minLeafPopulation, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"minLeafPopulation 只支持int类型参数, 传入类型为{type(minLeafPopulation)}",
            )

        invoke_args = {
            "numTrees": numTrees,
            "maxDepth": maxDepth,
            "maxNodes": maxNodes,
            "minLeafPopulation": minLeafPopulation,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "numTrees" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数numTrees不能为空")

        return FunctionHelper.apply(
            "Classifier.adaBoost", "engine.Classifier", invoke_args
        )

    @staticmethod
    def naiveBayes(smooth: [int, float] = 1.0) -> engine.Classifier:
        if smooth is not None and not isinstance(smooth, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"smooth 只支持(int,float)类型参数, 传入类型为{type(smooth)}",
            )

        invoke_args = {
            "smooth": smooth,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply(
            "Classifier.naiveBayes", "engine.Classifier", invoke_args
        )

    def confusionMatrix(self) -> engine.ConfusionMatrix:
        invoke_args = {
            "input": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply(
            "Classifier.confusionMatrix", "engine.ConfusionMatrix", invoke_args
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

import abc
import inspect
from typing import Union
from types import FunctionType

from aiearth import engine
from aiearth.engine.variable_node import VariableNode
from aiearth.engine.function_node import FunctionNode
from aiearth.engine.customfunction_node import CustomFunctionNode
from aiearth.engine.function_helper import FunctionHelper
from aiearth.core.error.aie_error import AIEError, AIEErrorCode


class Reducer(FunctionNode):
    def combine(
        self,
        reducer2: engine.Reducer,
        outputPrefix: str = "",
        sharedInputs: bool = False,
    ) -> engine.Reducer:
        if reducer2 is not None and not isinstance(reducer2, engine.Reducer):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"reducer2 只支持engine.Reducer类型参数, 传入类型为{type(reducer2)}",
            )

        if outputPrefix is not None and not isinstance(outputPrefix, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"outputPrefix 只支持str类型参数, 传入类型为{type(outputPrefix)}",
            )

        if sharedInputs is not None and not isinstance(sharedInputs, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"sharedInputs 只支持bool类型参数, 传入类型为{type(sharedInputs)}",
            )

        invoke_args = {
            "reducer1": self,
            "reducer2": reducer2,
            "outputPrefix": outputPrefix,
            "sharedInputs": sharedInputs,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "reducer2" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数reducer2不能为空")

        return FunctionHelper.apply("Reducer.combine", "engine.Reducer", invoke_args)

    @staticmethod
    def histogram(
        maxBuckets: int = None, minBucketWidth: [int, float] = None, maxRaw: int = None
    ) -> engine.Reducer:
        if maxBuckets is not None and not isinstance(maxBuckets, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"maxBuckets 只支持int类型参数, 传入类型为{type(maxBuckets)}",
            )

        if minBucketWidth is not None and not isinstance(minBucketWidth, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"minBucketWidth 只支持(int,float)类型参数, 传入类型为{type(minBucketWidth)}",
            )

        if maxRaw is not None and not isinstance(maxRaw, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"maxRaw 只支持int类型参数, 传入类型为{type(maxRaw)}"
            )

        invoke_args = {
            "maxBuckets": maxBuckets,
            "minBucketWidth": minBucketWidth,
            "maxRaw": maxRaw,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.histogram", "engine.Reducer", invoke_args)

    @staticmethod
    def count() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.count", "engine.Reducer", invoke_args)

    @staticmethod
    def max(numInputs: int = 1) -> engine.Reducer:
        if numInputs is not None and not isinstance(numInputs, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"numInputs 只支持int类型参数, 传入类型为{type(numInputs)}"
            )

        invoke_args = {
            "numInputs": numInputs,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.max", "engine.Reducer", invoke_args)

    @staticmethod
    def mean() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.mean", "engine.Reducer", invoke_args)

    @staticmethod
    def median(
        maxBuckets: int = None, minBucketWidth: [int, float] = None, maxRaw: int = None
    ) -> engine.Reducer:
        if maxBuckets is not None and not isinstance(maxBuckets, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"maxBuckets 只支持int类型参数, 传入类型为{type(maxBuckets)}",
            )

        if minBucketWidth is not None and not isinstance(minBucketWidth, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"minBucketWidth 只支持(int,float)类型参数, 传入类型为{type(minBucketWidth)}",
            )

        if maxRaw is not None and not isinstance(maxRaw, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"maxRaw 只支持int类型参数, 传入类型为{type(maxRaw)}"
            )

        invoke_args = {
            "maxBuckets": maxBuckets,
            "minBucketWidth": minBucketWidth,
            "maxRaw": maxRaw,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.median", "engine.Reducer", invoke_args)

    @staticmethod
    def min(numInputs: int = 1) -> engine.Reducer:
        if numInputs is not None and not isinstance(numInputs, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"numInputs 只支持int类型参数, 传入类型为{type(numInputs)}"
            )

        invoke_args = {
            "numInputs": numInputs,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.min", "engine.Reducer", invoke_args)

    @staticmethod
    def minMax() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.minMax", "engine.Reducer", invoke_args)

    @staticmethod
    def mode(
        maxBuckets: int = None, minBucketWidth: [int, float] = None, maxRaw: int = None
    ) -> engine.Reducer:
        if maxBuckets is not None and not isinstance(maxBuckets, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"maxBuckets 只支持int类型参数, 传入类型为{type(maxBuckets)}",
            )

        if minBucketWidth is not None and not isinstance(minBucketWidth, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"minBucketWidth 只支持(int,float)类型参数, 传入类型为{type(minBucketWidth)}",
            )

        if maxRaw is not None and not isinstance(maxRaw, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"maxRaw 只支持int类型参数, 传入类型为{type(maxRaw)}"
            )

        invoke_args = {
            "maxBuckets": maxBuckets,
            "minBucketWidth": minBucketWidth,
            "maxRaw": maxRaw,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.mode", "engine.Reducer", invoke_args)

    @staticmethod
    def product() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.product", "engine.Reducer", invoke_args)

    @staticmethod
    def stdDev() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.stdDev", "engine.Reducer", invoke_args)

    @staticmethod
    def sum() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.sum", "engine.Reducer", invoke_args)

    @staticmethod
    def variance() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.variance", "engine.Reducer", invoke_args)

    @staticmethod
    def allNonZero() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.allNonZero", "engine.Reducer", invoke_args)

    @staticmethod
    def anyNonZero() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.anyNonZero", "engine.Reducer", invoke_args)

    @staticmethod
    def bitwiseAnd() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.bitwiseAnd", "engine.Reducer", invoke_args)

    @staticmethod
    def bitwiseOr() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.bitwiseOr", "engine.Reducer", invoke_args)

    @staticmethod
    def sampleStdDev() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply(
            "Reducer.sampleStdDev", "engine.Reducer", invoke_args
        )

    @staticmethod
    def sampleVariance() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply(
            "Reducer.sampleVariance", "engine.Reducer", invoke_args
        )

    @staticmethod
    def histogram(buckets: int) -> engine.Reducer:
        if buckets is not None and not isinstance(buckets, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"buckets 只支持int类型参数, 传入类型为{type(buckets)}"
            )

        invoke_args = {
            "buckets": buckets,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "buckets" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数buckets不能为空")

        return FunctionHelper.apply("Reducer.histogram", "engine.Reducer", invoke_args)

    @staticmethod
    def fixedHistogram(
        min: [int, float], max: [int, float], buckets: int
    ) -> engine.Reducer:
        if min is not None and not isinstance(min, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"min 只支持(int,float)类型参数, 传入类型为{type(min)}"
            )

        if max is not None and not isinstance(max, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"max 只支持(int,float)类型参数, 传入类型为{type(max)}"
            )

        if buckets is not None and not isinstance(buckets, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"buckets 只支持int类型参数, 传入类型为{type(buckets)}"
            )

        invoke_args = {
            "min": min,
            "max": max,
            "buckets": buckets,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "min" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数min不能为空")

        if "max" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数max不能为空")

        if "buckets" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数buckets不能为空")

        return FunctionHelper.apply(
            "Reducer.fixedHistogram", "engine.Reducer", invoke_args
        )

    @staticmethod
    def linearFit() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Reducer.linearFit", "engine.Reducer", invoke_args)

    @staticmethod
    def linearRegression() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply(
            "Reducer.linearRegression", "engine.Reducer", invoke_args
        )

    @staticmethod
    def pearsonsCorrelation() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply(
            "Reducer.pearsonsCorrelation", "engine.Reducer", invoke_args
        )

    @staticmethod
    def spearmansCorrelation() -> engine.Reducer:
        invoke_args = {}

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply(
            "Reducer.spearmansCorrelation", "engine.Reducer", invoke_args
        )

    @staticmethod
    def ridgeRegression(regParam: [int, float] = 0.1) -> engine.Reducer:
        if regParam is not None and not isinstance(regParam, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"regParam 只支持(int,float)类型参数, 传入类型为{type(regParam)}",
            )

        invoke_args = {
            "regParam": regParam,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply(
            "Reducer.ridgeRegression", "engine.Reducer", invoke_args
        )

    @staticmethod
    def lassoRegression(regParam: [int, float] = 0.1) -> engine.Reducer:
        if regParam is not None and not isinstance(regParam, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"regParam 只支持(int,float)类型参数, 传入类型为{type(regParam)}",
            )

        invoke_args = {
            "regParam": regParam,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply(
            "Reducer.lassoRegression", "engine.Reducer", invoke_args
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

import abc
import inspect
from typing import Union
from types import FunctionType

from aiearth import engine
from aiearth.engine.variable_node import VariableNode
from aiearth.engine.function_node import FunctionNode
from aiearth.engine.customfunction_node import CustomFunctionNode
from aiearth.engine.function_helper import FunctionHelper
from aiearth.core.error.aie_error import AIEError, AIEErrorCode


class Kernel(FunctionNode):
    @staticmethod
    def chebyshev(
        radius: [int, float],
        units: str = "pixels",
        normalize: bool = False,
        magnitude: [int, float] = 1.0,
    ) -> engine.Kernel:
        if radius is not None and not isinstance(radius, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"radius 只支持(int,float)类型参数, 传入类型为{type(radius)}",
            )

        if units is not None and not isinstance(units, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"units 只支持str类型参数, 传入类型为{type(units)}"
            )

        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "radius": radius,
            "units": units,
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "radius" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数radius不能为空")

        return FunctionHelper.apply("Kernel.chebyshev", "engine.Kernel", invoke_args)

    @staticmethod
    def circle(
        radius: [int, float],
        units: str = "pixels",
        normalize: bool = False,
        magnitude: [int, float] = 1.0,
    ) -> engine.Kernel:
        if radius is not None and not isinstance(radius, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"radius 只支持(int,float)类型参数, 传入类型为{type(radius)}",
            )

        if units is not None and not isinstance(units, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"units 只支持str类型参数, 传入类型为{type(units)}"
            )

        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "radius": radius,
            "units": units,
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "radius" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数radius不能为空")

        return FunctionHelper.apply("Kernel.circle", "engine.Kernel", invoke_args)

    @staticmethod
    def compass(
        normalize: bool = False, magnitude: [int, float] = 1.0
    ) -> engine.Kernel:
        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Kernel.compass", "engine.Kernel", invoke_args)

    @staticmethod
    def cross(
        radius: [int, float],
        units: str = "pixels",
        normalize: bool = False,
        magnitude: [int, float] = 1.0,
    ) -> engine.Kernel:
        if radius is not None and not isinstance(radius, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"radius 只支持(int,float)类型参数, 传入类型为{type(radius)}",
            )

        if units is not None and not isinstance(units, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"units 只支持str类型参数, 传入类型为{type(units)}"
            )

        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "radius": radius,
            "units": units,
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "radius" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数radius不能为空")

        return FunctionHelper.apply("Kernel.cross", "engine.Kernel", invoke_args)

    @staticmethod
    def diamond(
        radius: [int, float],
        units: str = "pixels",
        normalize: bool = False,
        magnitude: [int, float] = 1.0,
    ) -> engine.Kernel:
        if radius is not None and not isinstance(radius, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"radius 只支持(int,float)类型参数, 传入类型为{type(radius)}",
            )

        if units is not None and not isinstance(units, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"units 只支持str类型参数, 传入类型为{type(units)}"
            )

        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "radius": radius,
            "units": units,
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "radius" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数radius不能为空")

        return FunctionHelper.apply("Kernel.diamond", "engine.Kernel", invoke_args)

    @staticmethod
    def euclidean(
        radius: [int, float],
        units: str = "pixels",
        normalize: bool = False,
        magnitude: [int, float] = 1.0,
    ) -> engine.Kernel:
        if radius is not None and not isinstance(radius, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"radius 只支持(int,float)类型参数, 传入类型为{type(radius)}",
            )

        if units is not None and not isinstance(units, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"units 只支持str类型参数, 传入类型为{type(units)}"
            )

        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "radius": radius,
            "units": units,
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "radius" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数radius不能为空")

        return FunctionHelper.apply("Kernel.euclidean", "engine.Kernel", invoke_args)

    @staticmethod
    def fixed(weights: list, normalize: bool = False) -> engine.Kernel:
        if weights is not None and not isinstance(weights, list):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"weights 只支持list类型参数, 传入类型为{type(weights)}"
            )

        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        invoke_args = {
            "weights": weights,
            "normalize": normalize,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "weights" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数weights不能为空")

        return FunctionHelper.apply("Kernel.fixed", "engine.Kernel", invoke_args)

    @staticmethod
    def gaussian(
        radius: [int, float],
        sigma: [int, float] = 1.0,
        units: str = "pixels",
        normalize: bool = False,
        magnitude: [int, float] = 1.0,
    ) -> engine.Kernel:
        if radius is not None and not isinstance(radius, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"radius 只支持(int,float)类型参数, 传入类型为{type(radius)}",
            )

        if sigma is not None and not isinstance(sigma, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"sigma 只支持(int,float)类型参数, 传入类型为{type(sigma)}"
            )

        if units is not None and not isinstance(units, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"units 只支持str类型参数, 传入类型为{type(units)}"
            )

        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "radius": radius,
            "sigma": sigma,
            "units": units,
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "radius" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数radius不能为空")

        return FunctionHelper.apply("Kernel.gaussian", "engine.Kernel", invoke_args)

    @staticmethod
    def kirsch(normalize: bool = False, magnitude: [int, float] = 1.0) -> engine.Kernel:
        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Kernel.kirsch", "engine.Kernel", invoke_args)

    @staticmethod
    def laplacian4(
        normalize: bool = False, magnitude: [int, float] = 1.0
    ) -> engine.Kernel:
        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Kernel.laplacian4", "engine.Kernel", invoke_args)

    @staticmethod
    def laplacian8(
        normalize: bool = False, magnitude: [int, float] = 1.0
    ) -> engine.Kernel:
        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Kernel.laplacian8", "engine.Kernel", invoke_args)

    @staticmethod
    def manhattan(
        radius: [int, float],
        units: str = "pixels",
        normalize: bool = False,
        magnitude: [int, float] = 1.0,
    ) -> engine.Kernel:
        if radius is not None and not isinstance(radius, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"radius 只支持(int,float)类型参数, 传入类型为{type(radius)}",
            )

        if units is not None and not isinstance(units, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"units 只支持str类型参数, 传入类型为{type(units)}"
            )

        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "radius": radius,
            "units": units,
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "radius" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数radius不能为空")

        return FunctionHelper.apply("Kernel.manhattan", "engine.Kernel", invoke_args)

    @staticmethod
    def plus(
        radius: [int, float],
        units: str = "pixels",
        normalize: bool = False,
        magnitude: [int, float] = 1.0,
    ) -> engine.Kernel:
        if radius is not None and not isinstance(radius, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"radius 只支持(int,float)类型参数, 传入类型为{type(radius)}",
            )

        if units is not None and not isinstance(units, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"units 只支持str类型参数, 传入类型为{type(units)}"
            )

        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "radius": radius,
            "units": units,
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "radius" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数radius不能为空")

        return FunctionHelper.apply("Kernel.plus", "engine.Kernel", invoke_args)

    @staticmethod
    def prewitt(
        normalize: bool = False, magnitude: [int, float] = 1.0
    ) -> engine.Kernel:
        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Kernel.prewitt", "engine.Kernel", invoke_args)

    @staticmethod
    def rectangle(
        xRadius: [int, float],
        yRadius: [int, float],
        units: str = "pixels",
        normalize: bool = False,
        magnitude: [int, float] = 1.0,
    ) -> engine.Kernel:
        if xRadius is not None and not isinstance(xRadius, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"xRadius 只支持(int,float)类型参数, 传入类型为{type(xRadius)}",
            )

        if yRadius is not None and not isinstance(yRadius, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"yRadius 只支持(int,float)类型参数, 传入类型为{type(yRadius)}",
            )

        if units is not None and not isinstance(units, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"units 只支持str类型参数, 传入类型为{type(units)}"
            )

        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "xRadius": xRadius,
            "yRadius": yRadius,
            "units": units,
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "xRadius" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数xRadius不能为空")

        if "yRadius" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数yRadius不能为空")

        return FunctionHelper.apply("Kernel.rectangle", "engine.Kernel", invoke_args)

    @staticmethod
    def roberts(
        normalize: bool = False, magnitude: [int, float] = 1.0
    ) -> engine.Kernel:
        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Kernel.roberts", "engine.Kernel", invoke_args)

    @staticmethod
    def sobel(normalize: bool = False, magnitude: [int, float] = 1.0) -> engine.Kernel:
        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Kernel.sobel", "engine.Kernel", invoke_args)

    @staticmethod
    def square(
        radius: [int, float],
        units: str = "pixels",
        normalize: bool = False,
        magnitude: [int, float] = 1.0,
    ) -> engine.Kernel:
        if radius is not None and not isinstance(radius, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"radius 只支持(int,float)类型参数, 传入类型为{type(radius)}",
            )

        if units is not None and not isinstance(units, str):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"units 只支持str类型参数, 传入类型为{type(units)}"
            )

        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        if magnitude is not None and not isinstance(magnitude, (int, float)):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"magnitude 只支持(int,float)类型参数, 传入类型为{type(magnitude)}",
            )

        invoke_args = {
            "radius": radius,
            "units": units,
            "normalize": normalize,
            "magnitude": magnitude,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "radius" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数radius不能为空")

        return FunctionHelper.apply("Kernel.square", "engine.Kernel", invoke_args)

    def add(self, kernel2: engine.Kernel, normalize: bool = False) -> engine.Kernel:
        if kernel2 is not None and not isinstance(kernel2, engine.Kernel):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"kernel2 只支持engine.Kernel类型参数, 传入类型为{type(kernel2)}",
            )

        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        invoke_args = {
            "kernel1": self,
            "kernel2": kernel2,
            "normalize": normalize,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "kernel2" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数kernel2不能为空")

        return FunctionHelper.apply("Kernel.add", "engine.Kernel", invoke_args)

    def subtract(
        self, kernel2: engine.Kernel, normalize: bool = False
    ) -> engine.Kernel:
        if kernel2 is not None and not isinstance(kernel2, engine.Kernel):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"kernel2 只支持engine.Kernel类型参数, 传入类型为{type(kernel2)}",
            )

        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        invoke_args = {
            "kernel1": self,
            "kernel2": kernel2,
            "normalize": normalize,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "kernel2" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数kernel2不能为空")

        return FunctionHelper.apply("Kernel.subtract", "engine.Kernel", invoke_args)

    def multiply(
        self, kernel2: engine.Kernel, normalize: bool = False
    ) -> engine.Kernel:
        if kernel2 is not None and not isinstance(kernel2, engine.Kernel):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"kernel2 只支持engine.Kernel类型参数, 传入类型为{type(kernel2)}",
            )

        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        invoke_args = {
            "kernel1": self,
            "kernel2": kernel2,
            "normalize": normalize,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "kernel2" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数kernel2不能为空")

        return FunctionHelper.apply("Kernel.multiply", "engine.Kernel", invoke_args)

    def divide(self, kernel2: engine.Kernel, normalize: bool = False) -> engine.Kernel:
        if kernel2 is not None and not isinstance(kernel2, engine.Kernel):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"kernel2 只支持engine.Kernel类型参数, 传入类型为{type(kernel2)}",
            )

        if normalize is not None and not isinstance(normalize, bool):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR,
                f"normalize 只支持bool类型参数, 传入类型为{type(normalize)}",
            )

        invoke_args = {
            "kernel1": self,
            "kernel2": kernel2,
            "normalize": normalize,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "kernel2" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数kernel2不能为空")

        return FunctionHelper.apply("Kernel.divide", "engine.Kernel", invoke_args)

    def inverse(self) -> engine.Kernel:
        invoke_args = {
            "kernel": self,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        return FunctionHelper.apply("Kernel.inverse", "engine.Kernel", invoke_args)

    def rotate(self, rotations: int) -> engine.Kernel:
        if rotations is not None and not isinstance(rotations, int):
            raise AIEError(
                AIEErrorCode.ARGS_ERROR, f"rotations 只支持int类型参数, 传入类型为{type(rotations)}"
            )

        invoke_args = {
            "kernel": self,
            "rotations": rotations,
        }

        invoke_args = {k: v for k, v in invoke_args.items() if v is not None}

        if "rotations" not in invoke_args:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "参数rotations不能为空")

        return FunctionHelper.apply("Kernel.rotate", "engine.Kernel", invoke_args)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from aiearth.core.client.endpoints import *
from .client import *
from aiearth.core import g_var
import time
from aiearth.core.error.aie_error import *
from aiearth import engine


class InteractiveSessionClient(BaseClient):
    class ResponseCode(object):
        OK = 0

    @staticmethod
    def post(url, reqeust_data, append_extra_hdrs=True):
        headers = {"Content-Type": "application/json"}
        resp = super(InteractiveSessionClient,
                     InteractiveSessionClient).post(url, headers, reqeust_data, append_extra_hdrs)
        response_data = resp.json()

        if "code" in response_data and response_data["code"] == InteractiveSessionClient.ResponseCode.OK:
            return response_data["payload"]
        else:
            extrainfo = {}
            if "taskId" in reqeust_data:
                extrainfo["taskId"] = reqeust_data["taskId"]
            raise AIEError(response_data["code"], response_data["msg"], json.dumps(extrainfo))

    @staticmethod
    def get(url, append_extra_hdrs=True):
        headers = {"Content-Type": "application/json"}
        resp = super(InteractiveSessionClient,
                     InteractiveSessionClient).get(url, headers, append_extra_hdrs)
        data = resp.json()
        if "code" in data and data["code"] == InteractiveSessionClient.ResponseCode.OK:
            return data["payload"]
        else:
            raise AIEError(data["code"], data["msg"])


class InteractiveSession():
    class SessionStatus(object):
        NOT_STARTED = "not_started"
        RECOVERING = "recovering"
        STARTING = "starting"

        IDLE = "idle"
        BUSY = "busy"
        SUCCESS = "success"

        SHUTTING_DOWN = "shutting_down"
        ERROR = "error"
        DEAD = "dead"
        KILLED = "killed"

    @staticmethod
    def create(enable_udf=False):
        data = {
            "enableUdf": "true" if enable_udf else "false"
        }

        url = Endpoints.INTERACTIVE_SESSION + InteractiveSessionResource.CREATE
        payload = InteractiveSessionClient.post(url, data, True)
        session_id = payload["id"]

        print("计算资源初始化中，请等待...", end="")
        status = InteractiveSession.status(session_id)
        while InteractiveSession.isStarting(status):
            # print(f"interactive session create status: {status}")
            time.sleep(5)
            print(".", end="")
            status = InteractiveSession.status(session_id)
        print("")
        if not InteractiveSession.isActive(status):
            raise AIEError(AIEErrorCode.ENVIRONMENT_INIT_ERROR,
                           f"计算资源初始化失败. status: {status}")
        g_var.set_var(
            g_var.GVarKey.InteractiveSession.INTERACTIVE_SESSION_ID, session_id)

    @staticmethod
    def status(session_id):
        url = Endpoints.INTERACTIVE_SESSION + \
            InteractiveSessionResource.QUERY.format(session_id)
        payload = InteractiveSessionClient.get(url)
        return payload

    @staticmethod
    def isStarting(status):
        return status["state"] in (InteractiveSession.SessionStatus.NOT_STARTED, InteractiveSession.SessionStatus.STARTING, InteractiveSession.SessionStatus.RECOVERING)

    @staticmethod
    def isActive(status):
        return status["state"] in (InteractiveSession.SessionStatus.IDLE, InteractiveSession.SessionStatus.BUSY, InteractiveSession.SessionStatus.SUCCESS)

    @staticmethod
    def isDead(status):
        return status["state"] in (InteractiveSession.SessionStatus.SHUTTING_DOWN, InteractiveSession.SessionStatus.ERROR, InteractiveSession.SessionStatus.DEAD, InteractiveSession.SessionStatus.KILLED)

    @staticmethod
    def active(session_id):
        status = InteractiveSession.status(session_id)
        return InteractiveSession.isActive(status)

    @staticmethod
    def getCurrentSessionId():
        if not g_var.has_var(g_var.GVarKey.InteractiveSession.INTERACTIVE_SESSION_ID):
            raise AIEError(AIEErrorCode.DEFAULT_INTERNAL_ERROR,
                           "", f'''服务端SessionID获取失败，请先调用
import aie
aie.Authenticate()
aie.Initialize() 
进行授权及初始化''')
        return g_var.get_var(g_var.GVarKey.InteractiveSession.INTERACTIVE_SESSION_ID)

    @staticmethod
    def cancelAllJobs():
        data = {
            "sessionId": InteractiveSession.getCurrentSessionId()
        }

        url = Endpoints.INTERACTIVE_SESSION + InteractiveSessionResource.CANCEL_ALL
        InteractiveSessionClient.post(url, data, True)

    @staticmethod
    def getInfo(obj):
        if isinstance(obj, (engine.Image, engine.Feature)) \
                and obj.func_name is None \
                and obj.invoke_args is None \
                and '_MAPPING_VAR' in obj.var_name:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "请勿在map函数中使用getInfo方法",
                           "请尝试使用for循环逐个获取，https://engine-aiearth.aliyun.com/docs/page/api?d=9e276c")
        expr = engine.serialize.serializer.encode(obj)
        options = {
            "taskType": "getInfo"
        }
        task_id = newUUID()
        data = {
            "taskId": task_id,
            "sessionId": InteractiveSession.getCurrentSessionId(),
            "script": "{\"expression\":" + expr + "}",
            "options": json.dumps(options)
        }

        url = Endpoints.INTERACTIVE_SESSION + InteractiveSessionResource.EXEC
        payload = InteractiveSessionClient.post(url, data)

        # print("payload", json.dumps(payload))

        if payload["state"] == "cancelled":
            raise AIEError(AIEErrorCode.JOB_CANCELLED, "该任务已取消")

        if payload["state"] != "success":
            extrainfo = {}
            if "taskId" in data:
                extrainfo["taskId"] = data["taskId"]
            raise AIEError(payload["output"]["error_code"],
                           payload["output"]["error_message"], json.dumps(extrainfo))
        return payload["output"]["data"]["application/json"]

    @staticmethod
    def getBounds(obj):
        if isinstance(obj, (engine.Image, engine.Feature)) \
                and obj.func_name is None \
                and obj.invoke_args is None \
                and '_MAPPING_VAR' in obj.var_name:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "请勿在map函数中使用getBounds方法",
                           "请尝试使用for循环逐个获取，https://engine-aiearth.aliyun.com/docs/page/api?d=9e276c")
        expr = engine.serialize.serializer.encode(obj)
        options = {
            "taskType": "getBounds"
        }
        task_id = newUUID()
        data = {
            "taskId": task_id,
            "sessionId": InteractiveSession.getCurrentSessionId(),
            "script": "{\"expression\":" + expr + "}",
            "options": json.dumps(options)
        }

        url = Endpoints.INTERACTIVE_SESSION + InteractiveSessionResource.EXEC
        payload = InteractiveSessionClient.post(url, data)

        # print("payload", json.dumps(payload))
        
        if payload["state"] == "cancelled":
            raise AIEError(AIEErrorCode.JOB_CANCELLED, "该任务已取消")

        if payload["state"] != "success":
            extrainfo = {}
            if "taskId" in data:
                extrainfo["taskId"] = data["taskId"]
            raise AIEError(payload["output"]["error_code"],
                           payload["output"]["error_message"], json.dumps(extrainfo))
        return payload["output"]["data"]["application/json"]

    @staticmethod
    def classifierTrain(obj):
        expr = engine.serialize.serializer.encode(obj)
        options = {
            "taskType": "getResult"
        }
        task_id = newUUID()
        data = {
            "taskId": task_id,
            "sessionId": InteractiveSession.getCurrentSessionId(),
            "script": "{\"expression\":" + expr + "}",
            "options": json.dumps(options)
        }

        url = Endpoints.INTERACTIVE_SESSION + InteractiveSessionResource.EXEC
        payload = InteractiveSessionClient.post(url, data)
        if payload["state"] == "cancelled":
            raise AIEError(AIEErrorCode.JOB_CANCELLED, "该任务已取消")

        if payload["state"] != "success":
            extrainfo = {}
            if "taskId" in data:
                extrainfo["taskId"] = data["taskId"]
            raise AIEError(payload["output"]["error_code"],
                           payload["output"]["error_message"], json.dumps(extrainfo))
        return payload["output"]["data"]["application/json"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import annotations

import os
import time
from typing import Union

from aiearth import engine
from .client.task import Task
from aiearth.core.error import AIEError, AIEErrorCode
from .client.dataset import Dataset
from aiearth.core.g_var import *
from aiearth.core.auth import Authenticate


class Export(object):
    class image(object):
        @staticmethod
        def toAsset(image: engine.Image, assetId: str, scale: int = 1000, config: dict = None, needPyramid: bool = True):
            if not isinstance(image, engine.Image):
                raise AIEError(AIEErrorCode.ARGS_ERROR,
                               f"image 只支持 aie.Image 类型参数, 传入类型为{type(image)}")
            if not isinstance(assetId, str):
                raise AIEError(AIEErrorCode.ARGS_ERROR,
                               f"assetId 只支持 str 类型参数, 传入类型为{type(image)}")

            options = {
                "taskType": "ExportImage",
                "assetId": assetId,
                "scale": scale,
                "function": "image.toAsset",
                "needPyramid": needPyramid
            }

            if config is not None:
                options["config"] = config
            return Task(image, options)

    class feature(object):
        @staticmethod
        def toAsset(featureCollection: engine.FeatureCollection, assetId: str, config: dict = None):
            if not isinstance(featureCollection, engine.FeatureCollection):
                raise AIEError(AIEErrorCode.ARGS_ERROR,
                               f"featureCollection 只支持 aie.FeatureCollection 类型参数, 传入类型为{type(featureCollection)}")
            if not isinstance(assetId, str):
                raise AIEError(AIEErrorCode.ARGS_ERROR,
                               f"assetId 只支持 str 类型参数, 传入类型为{type(featureCollection)}")

            options = {
                "taskType": "ExportFeature",
                "assetId": assetId,
                "function": "feature.toAsset"
            }

            if config is not None:
                options["config"] = config
            return Task(featureCollection, options)

    class dataset(object):
        SUPPORT_SUFFIX = ["csv", "xls", "xlsx", "txt", "md", "jar", "png", "jpg", "jpeg",
                          "mp4", "avi", "acc", "mp3", "wmv", "h5", "bin", "pdf", "html", "json", "zip"]

        @staticmethod
        def toAsset(name: str, args: Union[str, list]):
            if not (has_var(GVarKey.Authenticate.CLIENT_ID) and get_var(GVarKey.Authenticate.CLIENT_ID) == Authenticate.ClientId.ALIYUN_JUPYTER):
                raise AIEError(AIEErrorCode.ARGS_ERROR,
                               f"aie.Export.feature.toAsset 只支持在AIE云平台jupyter环境里面调用")

            if not isinstance(name, str):
                raise AIEError(AIEErrorCode.ARGS_ERROR,
                               f"name 只支持 str 类型参数, 传入类型为{type(name)}")

            files = args
            if isinstance(args, str):
                files = [args]

            if not all(isinstance(file, str) for file in files):
                raise AIEError(AIEErrorCode.ARGS_ERROR,
                               f"args 只支持 str|list[str] 类型参数, 传入类型为{type(args)}")

            files_with_suffix = []
            for file in files:
                if not os.path.exists(file):
                    raise AIEError(AIEErrorCode.ARGS_ERROR,
                                   f"文件{file}路径不存在")

                _, suffix = os.path.splitext(file)
                suffix = suffix[1:]
                if suffix not in Export.dataset.SUPPORT_SUFFIX:
                    raise AIEError(AIEErrorCode.ARGS_ERROR,
                                   f"文件类型{suffix}不支持，支持的文件后缀类型为:{Export.dataset.SUPPORT_SUFFIX}")

                files_with_suffix.append((file, suffix))

            dataset = Dataset(name)
            for file, suffix in files_with_suffix:
                print(f"上传文件中: {file}")
                dataset.upload(file, suffix)

            dataset.publish()
            print("正在导出数据集...")
            while(dataset.active()):
                time.sleep(5)

            print(dataset.status())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import requests
from aiearth.core.client.endpoints import newUUID, Endpoints, SdkGatewayResource
from .client import Client
from aiearth import engine
from aiearth.core.error.aie_error import *
from aiearth.engine.serialize import serializer


class Maps(object):
    class AddLayerType:
        Image = "Image"
        Feature = "Feature"

    @staticmethod
    def getMapId(obj, vis_params=None):
        request_id = newUUID()

        add_layer_type = None
        if isinstance(obj, engine.Image):
            add_layer_type = Maps.AddLayerType.Image
        elif isinstance(obj, engine.ImageCollection):
            obj = obj.mosaic()
            add_layer_type = Maps.AddLayerType.Image
        elif isinstance(obj, engine.Geometry):
            add_layer_type = Maps.AddLayerType.Feature
        elif isinstance(obj, engine.Feature):
            add_layer_type = Maps.AddLayerType.Feature
        elif isinstance(obj, engine.FeatureCollection):
            add_layer_type = Maps.AddLayerType.Feature

        expr = serializer.encode(obj)

        options = {
            "mapId": newUUID(),
            "taskType": "addLayer",
            "addLayerType": add_layer_type,
            "visParams": vis_params
        }

        options_serialize = ""
        try: 
            options_serialize = json.dumps(options, ensure_ascii=False, allow_nan=False)
        except Exception as e:
            raise AIEError(AIEErrorCode.ARGS_ERROR, "请检查visParams参数", str(e))
        data = {"requestId": request_id,
                "expr": expr,
                "options": options_serialize}

        url = Endpoints.SDK_GATEWAY + SdkGatewayResource.Maps.GET_MAP_ID
        return Client.post(url, data)
